@@ -1,4 +1,4 @@
-// $Id: Banner.cc 3491 2009-09-04 21:40:05Z flaterco $
+// $Id: Banner.cc 4334 2012-02-19 20:50:35Z flaterco $
 
 /*  Banner  Graph printed sideways on tractor feed dot matrix printer
 
@@ -20,6 +20,7 @@
 
 #include "common.hh"
 #include "Graph.hh"
+#include "PixelatedGraph.hh"
 #include "TTYGraph.hh"
 #include "Banner.hh"
 
@@ -78,17 +79,22 @@ Banner * const Banner::factory (const Station &station,
 }
 
 
-void Banner::drawHorizontalLine (int xlo,
-                                 int xhi,
-                                 int y,
+void Banner::drawHorizontalLineP (int xlo, int xhi, int y,
 			         Colors::Colorchoice c unusedParameter) {
+  const char bar (VT100_mode ? '' : '|');
   for (int i=xlo; i<=xhi; ++i)
-    setPixel (i, y, '|');
+    setPixel (i, y, bar);
 }
 
 
-void Banner::drawHourTick (int x, Colors::Colorchoice c unusedParameter) {
-  setPixel (x, _ySize-1, '-');
+void Banner::drawHourTick (double x, Colors::Colorchoice c unusedParameter,
+bool thick) {
+  int ix = Global::ifloor(x);
+  setPixel (ix, _ySize-1, '-');
+  if (thick) {
+    setPixel (ix-1, _ySize-1, '-');
+    setPixel (ix+1, _ySize-1, '-');
+  }
 }
 
 
@@ -103,6 +109,8 @@ void Banner::print (Dstr &text_out) {
       buf[y] = tty[(_ySize-1-y)*_xSize+x];
     text_out += &(buf[0]);
   }
+  if (VT100_mode)
+    VT100_postproc (text_out);
 }
 
 
@@ -117,21 +125,22 @@ void Banner::drawStringSideways (int x, int y, const Dstr &s) {
 }
 
 
-void Banner::labelHourTick (int x, const Dstr &label) {
-  drawStringSideways (x,
+void Banner::labelHourTick (double x, const Dstr &label) {
+  drawStringSideways (Global::ifloor(x),
 		      _ySize - hourTickLen() - 1,
 		      label);
 }
 
 
 void Banner::labelEvent (int topLine, const EventBlurb &blurb) {
+  int x = Global::ifloor (blurb.x);
   if (topLine >= 0) {
-    drawStringSideways (blurb.x-1, blurb.line1.length()-1, blurb.line1);
-    drawStringSideways (blurb.x,   blurb.line2.length()-1, blurb.line2);
+    drawStringSideways (x-1, blurb.line1.length()-1, blurb.line1);
+    drawStringSideways (x,   blurb.line2.length()-1, blurb.line2);
   } else {
     int y (_ySize - hourTickLen() - 4);
-    drawStringSideways (blurb.x-1, y, blurb.line1);
-    drawStringSideways (blurb.x,   y, blurb.line2);
+    drawStringSideways (x-1, y, blurb.line1);
+    drawStringSideways (x,   y, blurb.line2);
   }
 }
 
@@ -151,5 +160,3 @@ void Banner::drawTitleLine (const Dstr &title unusedParameter) {
 const bool Banner::isBanner() const {
   return true;
 }
-
-// Cleanup2006 Done
