@@ -1,4 +1,4 @@
-// $Id: Station.cc 3491 2009-09-04 21:40:05Z flaterco $
+// $Id: Station.cc 4340 2012-02-20 03:31:31Z flaterco $
 
 /*  Station  A tide station.
 
@@ -27,10 +27,12 @@
 #include "common.hh"
 #include "Calendar.hh"
 #include "Graph.hh"
+#include "PixelatedGraph.hh"
 #include "TTYGraph.hh"
 #include "Banner.hh"
 #include "RGBGraph.hh"
 #include "Skycal.hh"
+#include "SVGGraph.hh"
 
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
@@ -646,6 +648,7 @@ void Station::aboutMode (Dstr &text_out,
   if (form == Format::HTML)
     text_out = "<table>\n";
   else {
+    text_out = (codeset == "VT100" ? Global::VT100_init : (char*)NULL);
     MetaFieldVector::const_iterator it = _metadata.begin();
     while (it != _metadata.end()) {
       if (it->name.length() > maximumNameLength)
@@ -658,11 +661,18 @@ void Station::aboutMode (Dstr &text_out,
     if (form == Format::HTML) {
       text_out += "<tr><td valign=top>";
       text_out += it->name;
-      text_out += "</td><td valign=top><pre>";
-      text_out += it->value;
-      text_out += "</pre></td></td>\n";
+      text_out += "</td><td valign=top><font face=\"monospace\">";
+      Dstr temp (it->value);
+      temp.repstr ("\n", "<br>\n");
+      text_out += temp;
+      text_out += "</font></td></td>\n";
     } else {
       Dstr tmp1 (it->name), tmp2 (it->value), tmp3;
+      // Ignore Global::degreeSign if codeset was overridden.
+      if (codeset == "VT100" && (tmp1 == "Coordinates" ||
+				 tmp1 == "Flood direction" ||
+				 tmp1 == "Ebb direction"))
+        tmp2.repstr ("", Global::degreeSign);
       tmp1.pad (maximumNameLength+2);
       tmp2.getline (tmp3);
       tmp1 += tmp3;
@@ -726,6 +736,8 @@ CALSCALE:GREGORIAN\r\n\
 METHOD:PUBLISH\r\n";
   } else {
 
+    if (form == Format::text && Global::codeset == "VT100")
+      text_out += Global::VT100_init;
     if (form == Format::HTML)
       text_out += "<h3>";
     text_out += name;
@@ -737,6 +749,8 @@ METHOD:PUBLISH\r\n";
     else {
       Dstr t;
       coordinates.print (t);
+      if (form == Format::text && Global::needDegrees())
+        t.repstr ("", Global::degreeSign);
       text_out += t;
       text_out += '\n';
     }
@@ -749,6 +763,8 @@ METHOD:PUBLISH\r\n";
       text_out += "Flood direction ";
       Dstr tmpbuf;
       maxCurrentBearing.print (tmpbuf);
+      if (form == Format::text && Global::needDegrees())
+        tmpbuf.repstr ("", Global::degreeSign);
       text_out += tmpbuf;
       text_out += '\n';
     }
@@ -758,6 +774,8 @@ METHOD:PUBLISH\r\n";
       text_out += "Ebb direction ";
       Dstr tmpbuf;
       minCurrentBearing.print (tmpbuf);
+      if (form == Format::text && Global::needDegrees())
+        tmpbuf.repstr ("", Global::degreeSign);
       text_out += tmpbuf;
       text_out += '\n';
     }
@@ -999,19 +1017,52 @@ void Station::bannerMode (Dstr &text_out,
 
 
 void Station::graphMode (Dstr &text_out,
-                         Timestamp startTime) {
-  TTYGraph g (Global::settings["tw"].u, Global::settings["th"].u);
-  g.drawTides (this, startTime);
-  g.print (text_out);
+                         Timestamp startTime,
+			 Format::Format form) {
+  switch (form) {
+    // print method is not in Graph / don't need yet another class
+  case Format::text:
+    {
+      TTYGraph g (Global::settings["tw"].u, Global::settings["th"].u);
+      g.drawTides (this, startTime);
+      g.print (text_out);
+    }
+    break;
+  case Format::SVG:
+    {
+      SVGGraph g (Global::settings["gw"].u, Global::settings["gh"].u);
+      g.drawTides (this, startTime);
+      g.print (text_out);
+    }
+    break;
+  default:
+    assert (false);
+  }
 }
 
 
-void Station::clockMode (Dstr &text_out) {
-  TTYGraph g (Global::settings["tw"].u,
-	      Global::settings["th"].u,
-	      Graph::clock);
-  g.drawTides (this, (time_t)time(NULL));
-  g.print (text_out);
+void Station::clockMode (Dstr &text_out, Format::Format form) {
+  switch (form) {
+    // print method is not in Graph / don't need yet another class
+  case Format::text:
+    {
+      TTYGraph g (Global::settings["tw"].u, Global::settings["th"].u,
+		  Graph::clock);
+      g.drawTides (this, (time_t)time(NULL));
+      g.print (text_out);
+    }
+    break;
+  case Format::SVG:
+    {
+      SVGGraph g (Global::settings["cw"].u, Global::settings["gh"].u,
+		  Graph::clock);
+      g.drawTides (this, (time_t)time(NULL));
+      g.print (text_out);
+    }
+    break;
+  default:
+    assert (false);
+  }
 }
 
 
@@ -1103,7 +1154,8 @@ void Station::print (Dstr &text_out,
       Global::log ("Can't happen:  Station::print called for graph mode, PNG form:  use graphModePNG instead\n", LOG_ERR);
       assert (false);
     case Format::text:
-      graphMode (text_out, startTime);
+    case Format::SVG:
+      graphMode (text_out, startTime, form);
       break;
     default:
       Global::formatBarf (mode, form);
@@ -1116,7 +1168,8 @@ void Station::print (Dstr &text_out,
       Global::log ("Can't happen:  Station::print called for clock mode, PNG form:  use graphModePNG instead\n", LOG_ERR);
       assert (false);
     case Format::text:
-      clockMode (text_out);
+    case Format::SVG:
+      clockMode (text_out, form);
       break;
     default:
       Global::formatBarf (mode, form);
