@@ -1,4 +1,4 @@
-// $Id: StationIndex.cc 2641 2007-09-02 21:31:02Z flaterco $
+// $Id: StationIndex.cc 4330 2012-02-19 01:15:53Z flaterco $
 
 /*  StationIndex  Collection of StationRefs.
 
@@ -238,7 +238,7 @@ void StationIndex::print (Dstr &text_out,
       if (namewidth < 10)
 	namewidth = 10;
       char fmt[80];
-      sprintf (fmt, "%%-%d.%ds %%-3.3s %%-23.23s\n", namewidth, namewidth);
+      sprintf (fmt, "%%-%d.%ds %%-3.3s %%s\n", namewidth, namewidth);
       SafeVector<char> buf (tw + 30);
       for (unsigned long i=0; i<size(); ++i) {
 	Dstr styp, c;
@@ -247,12 +247,14 @@ void StationIndex::print (Dstr &text_out,
 	else
 	  styp = "Sub";
 	operator[](i)->coordinates.print (c, Coordinates::fixedWidth);
-	sprintf (&(buf[0]),
+        if (Global::needDegrees())
+          c.repstr ("", Global::degreeSign);
+	sprintf (&buf[0],
                  fmt,
                  operator[](i)->name.aschar(),
                  styp.aschar(),
                  c.aschar());
-	text_out += &(buf[0]);
+	text_out += &buf[0];
       }
     }
     break;
