@@ -1,4 +1,4 @@
-// $Id: Timestamp.cc 2871 2007-12-09 14:33:44Z flaterco $
+// $Id: Timestamp.cc 4406 2012-03-04 14:26:16Z flaterco $
 
 // Timestamp:  A point in time.  See also Year, Date, Interval.
 
@@ -237,6 +237,11 @@ static ZoneinfoSupportLevel zoneinfoSupportLevel (TZNULL);
    2004-09-16
    Added :America/Juneau, :America/Nome, :America/Yakutat and
    :Pacific/Johnston.
+
+   2012-02-21
+   Changed :Pacific/Apia from SST11 to WST-13.
+   Doubtless there is worse code rot than that, but the correct fix is
+   to delete this entire workaround, not to try to update it.
 */
 
 /* In many cases, these substitutions will either break DST adjustments
@@ -574,7 +579,7 @@ static constString subs[][4] = {
 {":Indian/Mayotte",             "EAT-3", "EAT-3", "EAT-3"},
 {":Indian/Reunion",             "SMT-4", "SMT-4", "SMT-4"},
 
-{":Pacific/Apia",               "SST11", "SST11", "SST11"},
+{":Pacific/Apia",               "WST-13", "WST-13", "WST-13"},
 {":Pacific/Auckland",           ":NZ", "NZST-12NZDT", "NST-12"},
 {":Pacific/Chatham",            "CST-12:45", "CST-12:45", "CST-12:45"},
 {":Pacific/Easter",             ":Chile/EasterIsland", "CST6", "CST6"},
@@ -1188,12 +1193,14 @@ Timestamp::Timestamp (const Dstr &timeString, const Dstr &timezone) {
   tempTm.tm_sec = 0;
   // Deliberately using %u instead of %d so that dashes won't be
   // interpreted as minus signs.  This causes compiler warnings.
+  quashWarning(-Wformat)
   if (sscanf (timeString.aschar(), "%u-%u-%u %u:%u",
     &(tempTm.tm_year),
     &(tempTm.tm_mon),
     &(tempTm.tm_mday),
     &(tempTm.tm_hour),
     &(tempTm.tm_min)) != 5) {
+  unquashWarning
     Dstr details ("The offending time specification was ");
     details += timeString;
     Global::barf (Error::BAD_TIMESTAMP, details);
