@@ -1,4 +1,4 @@
-// $Id: tide.cc 2946 2008-01-18 23:12:25Z flaterco $
+// $Id: tide.cc 4327 2012-02-18 22:55:54Z flaterco $
 
 /*  tide  Command-line client for dusty old TTYs and line printers.
 
@@ -152,7 +152,7 @@ static void loopLocations (FILE *outfp) {
 
 int main (int argc, char **argv) {
   srand (time (NULL));
-  Global::initCodeset();
+  Global::initTerm();
   Global::settings.applyUserDefaults();
   Global::settings.applyCommandLine (argc, argv);
   Global::settings.fixUpDeprecatedSettings();
@@ -194,9 +194,9 @@ Other switches:\n\
       Specify the begin (start) time for predictions.\n\
   -e \"YYYY-MM-DD HH:MM\"\n\
       Specify the end (stop) time for predictions.\n\
-  -f c|h|i|l|p|t\n\
-      Specify the output format as CSV, HTML, iCalendar, LaTeX, PNG, or text.\n\
-      The default is text.\n\
+  -f c|h|i|l|p|t|v\n\
+      Specify the output format as CSV, HTML, iCalendar, LaTeX, PNG, text, or\n\
+      SVG.  The default is text.\n\
   -m a|b|c|C|g|k|l|m|p|r|s\n\
       Specify mode to be about, banner, calendar, alt. calendar, graph, clock,\n\
       list, medium rare, plain, raw, or stats.  The default is plain.\n\
