@@ -1,4 +1,4 @@
-// $Id: xtide.cc 2641 2007-09-02 21:31:02Z flaterco $
+// $Id: xtide.cc 4327 2012-02-18 22:55:54Z flaterco $
 
 /*  XTide  Harmonic tide clock and tide predictor.
 
@@ -23,10 +23,8 @@
 
 int main (int argc, char **argv) {
   srand (time (NULL));
-  Global::initCodeset();
+  Global::initTerm();
   Global::root = new xxRoot (argc, argv);
   Global::root->run();
   exit (0);
 }
-
-// Cleanup2006 Done
