@@ -1,4 +1,4 @@
-// $Id: xttpd.cc 2932 2008-01-10 00:56:57Z flaterco $
+// $Id: xttpd.cc 4359 2012-02-22 03:10:35Z flaterco $
 
 /*  xttpd  XTide web server.
 
@@ -20,7 +20,9 @@
 
 #include "common.hh"
 #include "Graph.hh"
+#include "PixelatedGraph.hh"
 #include "RGBGraph.hh"
+#include "SVGGraph.hh"
 #include "ZoneIndex.hh"
 
 #include <sys/utsname.h>
@@ -496,11 +498,14 @@ static void loadLocationPage (int s, const Dstr &filename, unsigned long i) {
   StationIndex &stationIndex (Global::stationIndex());
   Station *station = stationIndex[i]->load();
 
+  // 20110730  IE9 plays dumb on SVG if <!DOCTYPE html> is not present.
+
   Dstr pageText ("\
 HTTP/1.0 200 OK\n\
 MIME-version: 1.0\n\
 Content-type: text/html\n\
 \n\
+<!DOCTYPE html>\n\
 <html>\n\
 <head><title> ");
 
@@ -536,15 +541,32 @@ database.&nbsp; Summer Time (Daylight Savings Time) adjustments will not be done
 for some locations. </p>\n";
   }
 
-  pageText += "<p><img src=\"/graphs/";
+  // Query browser for SVG support; fall back to PNG if not supported.
+  Dstr repeatText (timeControlString);
+  repeatText += "\\\" width=";
+  repeatText += Global::settings["gw"].u;
+  repeatText += " height=";
+  repeatText += Global::settings["gh"].u;
+  repeatText += " /></p>\");\n";
+  pageText += "<script type=\"text/javascript\">\n";
+  pageText += "if (document.implementation.hasFeature(\"http://www.w3.org/TR/SVG11/feature#BasicStructure\", \"1.1\")) {\n";
+  // 20110730    Firefox 5     Internet Explorer 9
+  // img        Ugly scaling           OK
+  // object         OK                FAIL
+  // embed          OK                 OK
+  pageText += "\tdocument.write(\"<p><embed type=\\\"image/svg+xml\\\" alt=\\\"SVG graph\\\" src=\\\"/graphs/";
+  pageText += i;
+  pageText += ".svg";
+  pageText += repeatText;
+  pageText += "} else {\n";
+  pageText += "\tdocument.write(\"<p><img alt=\\\"PNG graph\\\" src=\\\"/graphs/";
   pageText += i;
   pageText += ".png";
-  pageText += timeControlString;
-  pageText += "\" width=";
-  pageText += Global::settings["gw"].u;
-  pageText += " height=";
-  pageText += Global::settings["gh"].u;
-  pageText += "></p><p><pre>\n";
+  pageText += repeatText;
+  pageText += "}\n";
+  pageText += "</script>\n";
+
+  pageText += "<p><pre>\n";
 
   station->plainMode (text_out,
                       startTime,
@@ -841,12 +863,17 @@ static void writePNGToSocket (png_structp png_ptr unusedParameter,
 }
 
 
-static void graphImage (int s, const Dstr &filename) {
+static void graphImage (int s, const Dstr &filename) 
+{
   unsigned long i;
-  if (sscanf (filename.aschar(), "/graphs/%lu.png", &i) != 1) {
+
+  // Failure to match the stuff following the %lu has no observable impact,
+  // so it doesn't help to put %lu.png or %lu.svg.
+  if (sscanf (filename.aschar(), "/graphs/%lu", &i) != 1) {
     badClientLocation (filename);
     notFoundBarf (s);
   }
+  bool isSVG = filename.contains(".svg");
 
   StationIndex &stationIndex (Global::stationIndex());
   if (i >= stationIndex.size()) {
@@ -861,13 +888,27 @@ static void graphImage (int s, const Dstr &filename) {
     timeControlString, form);
 
   Dstr pageText
-    ("HTTP/1.0 200 OK\nMIME-version: 1.0\nPragma: no-cache\nContent-type: image/png\n\n");
+     ("HTTP/1.0 200 OK\nMIME-version: 1.0\nPragma: no-cache\n");
   checkedWrite (s, pageText.aschar(), pageText.length());
 
-  RGBGraph g (Global::settings["gw"].u, Global::settings["gh"].u);
-  g.drawTides (station, startTime);
-  PNGSocket = s;
-  g.writeAsPNG (writePNGToSocket);
+  if (!isSVG) {
+     Dstr contentType ("Content-type: image/png\n\n");
+     checkedWrite (s, contentType.aschar(), contentType.length());
+     
+     RGBGraph g (Global::settings["gw"].u, Global::settings["gh"].u);
+     g.drawTides (station, startTime);
+     PNGSocket = s;
+     g.writeAsPNG (writePNGToSocket);
+  } else {
+     Dstr contentType ("Content-type: image/svg+xml\n\n");
+     checkedWrite (s, contentType.aschar(), contentType.length());
+     SVGGraph svg (Global::settings["gw"].u, Global::settings["gh"].u);
+     Dstr text_out;
+     svg.drawTides (station, startTime);
+     svg.print (text_out);
+     checkedWrite (s, text_out.aschar(), text_out.length());
+  }
+
   close (s);
   exit (0);
 }
@@ -1186,7 +1227,7 @@ int main (int argc, char **argv) {
       }
     }
     // Clean up zombies.
-    while (waitpid (-1, NULL, WNOHANG|WUNTRACED) > 0);
+    while (waitpid (-1, (int*)NULL, WNOHANG|WUNTRACED) > 0);
   }
 
   exit (0);
