@@ -1,4 +1,4 @@
-// $Id: xxClock.cc 3493 2009-09-11 12:16:11Z flaterco $
+// $Id: xxClock.cc 4497 2012-05-27 00:33:09Z flaterco $
 
 /*  xxClock  Tide clock.
 
@@ -20,9 +20,13 @@
 
 #include "xtide.hh"
 #include "Graph.hh"
+#include "PixelatedGraph.hh"
 #include "RGBGraph.hh"
+#include "SVGGraph.hh"
 #include "xxPixmapGraph.hh"
-#include "xxSimplePrompt.hh"
+#include "xxHorizDialog.hh"
+#include "xxMultiChoice.hh"
+#include "xxReconfigurable.hh"
 #include "xxFilename.hh"
 
 
@@ -43,7 +47,7 @@ bring back the buttons, click on the graph again.\n\
 \n\
 The options menu supplies the following commands:\n\
 \n\
-  Save:  Use this to write the tide predictions to a PNG image file.\n\
+  Save:  Use this to write the tide predictions to an image file.\n\
 \n\
   Set Mark:  This is used to set the 'mark level' for tide predictions.\n\
   In clock mode, this only causes a line to be drawn at the mark level.\n\
@@ -115,18 +119,38 @@ static void exposureHandler (Widget w unusedParameter,
 
 
 void xxClock::save () {
-  (void) new xxFilename (*popup, *this, "tides.png");
+  (void) new xxFilename (*popup, *this, false);
 }
 
 
-void xxClock::save (const Dstr &filename) {
-  if ((Global::PNGFile = fopen (filename.aschar(), "wb"))) {
-    RGBGraph g (curGraphWidth, curGraphHeight, Graph::clock);
-    g.drawTides (_station.get(), t);
-    g.writeAsPNG (Global::writePNGToFile);
-    fclose (Global::PNGFile);
-  } else
-    Global::cantOpenFile (filename, Error::nonfatal);
+void xxClock::save (const Dstr &filename, Format::Format form) {
+  switch (form) {
+  case Format::PNG:
+    if ((Global::PNGFile = fopen (filename.aschar(), "wb"))) {
+      RGBGraph g (curGraphWidth, curGraphHeight, Graph::clock);
+      g.drawTides (_station.get(), t);
+      g.writeAsPNG (Global::writePNGToFile);
+      fclose (Global::PNGFile);
+    } else
+      Global::cantOpenFile (filename, Error::nonfatal);
+    break;
+  case Format::SVG:
+    {
+      FILE *fp;
+      if ((fp = fopen (filename.aschar(), "w"))) {
+	Dstr text;
+	SVGGraph g (curGraphWidth, curGraphHeight, Graph::clock);
+	g.drawTides (_station.get(), t);
+	g.print (text);
+	fprintf (fp, "%s", text.aschar());
+	fclose (fp);
+      } else
+	Global::cantOpenFile (filename, Error::nonfatal);
+    }
+    break;
+  default:
+    assert (false);
+  }
 }
 
 
@@ -166,9 +190,27 @@ static void xxClockTimerCallback (XtPointer client_data,
 }
 
 
+void xxClock::updatePixmap (bool resized) {
+  if (xxX::displaySucks) {
+    if (resized)
+      graph = std::auto_ptr<xxPixmapGraph> (new xxPixmapGraph (curGraphWidth,
+							       curGraphHeight,
+							       Graph::clock));
+    graph->drawTides (_station.get(), t, &analogAngle);
+  } else {
+    RGBGraph g (curGraphWidth, curGraphHeight, Graph::clock);
+    g.drawTides (_station.get(), t, &analogAngle);
+    if (havePixmap)
+      XFreePixmap (xxX::display, pixmap);
+    havePixmap = true;
+    pixmap = xxX::makePixmap (curGraphWidth, curGraphHeight, g.rgb);
+  }
+}
+
+
 void xxClock::clockTick() {
   t = (time_t)time(NULL);
-  graph->drawTides (_station.get(), t, &analogAngle);
+  updatePixmap (false);
   draw();
   redrawIcon();
   resetTimer();
@@ -197,6 +239,7 @@ xxClock::xxClock (const xxWidget &shell,
   _buttonsStyle(buttonsStyle),
   iconWindowToggle(true) {
 
+  havePixmap = false;
   Dstr title (_station->name);
   title += " (Clock)";
   setTitle (title);
@@ -232,7 +275,7 @@ xxClock::xxClock (const xxWidget &shell,
       {XtNinternalHeight, (XtArgVal)0},
       {XtNinternalWidth, (XtArgVal)0}
     };
-    Widget labelWidget = XtCreateManagedWidget ("",
+    Widget labelWidget = xxX::createXtWidget ("",
       labelWidgetClass, container->widget(), labelArgs, 6);
     label = xxX::wrap (labelWidget);
     XtAddEventHandler (labelWidget, ButtonPressMask, False,
@@ -305,10 +348,10 @@ void xxClock::redrawIcon() {
   XDrawArc (xxX::display, clockIcon, xxX::iconTextGC,
 	    c-1, c-1, 2, 2, 0, 360*64);
   XDrawString (xxX::display, clockIcon, xxX::iconTextGC,
-	       c-(xxX::stringWidth(xxX::tinyFontStruct, hicap)/2), 7,
+	       c-(xxX::stringWidth(xxX::iconFontStruct, hicap)/2), 7,
 	       hicap, strlen(hicap));
   XDrawString (xxX::display, clockIcon, xxX::iconTextGC,
-	       c-(xxX::stringWidth(xxX::tinyFontStruct, locap)/2),
+	       c-(xxX::stringWidth(xxX::iconFontStruct, locap)/2),
 	       xxX::iconSize, locap, strlen(locap));
   int handx (Global::iround(c + hl * sin (analogAngle)));
   int handy (Global::iround(c - hl * cos (analogAngle)));
@@ -337,10 +380,7 @@ void xxClock::redrawIcon() {
 
 
 void xxClock::redraw() {
-  graph = std::auto_ptr<xxPixmapGraph> (new xxPixmapGraph (curGraphWidth,
-							   curGraphHeight,
-							   Graph::clock));
-  graph->drawTides (_station.get(), t, &analogAngle);
+  updatePixmap (true);
   if (_buttonsStyle == buttons) {
     Arg labelArgs[2] =  {
       {XtNheight, (XtArgVal)curGraphHeight},
@@ -353,11 +393,11 @@ void xxClock::redraw() {
 
 
 void xxClock::draw (int x, int y, int width, int height) {
-  if (graph.get() && _isRealized) {
+  if ((havePixmap || graph.get()) && _isRealized) {
     Widget target = (_buttonsStyle == noButtons ? popup->widget()
 		                                : label->widget());
     XCopyArea (xxX::display,
-	       graph->pixmap,
+	       (havePixmap ? pixmap : graph->pixmap),
 	       XtWindow(target),
 	       xxX::textGC,
 	       x,
@@ -386,6 +426,8 @@ void xxClock::draw () {
 xxClock::~xxClock() {
   XtRemoveTimeOut (timer);
   unrealize();
+  if (havePixmap)
+    XFreePixmap (xxX::display, pixmap);
   XDestroyWindow (xxX::display, iconWindowLocal);
   XFreePixmap (xxX::display, clockIcon);
 }
