@@ -1,4 +1,4 @@
-// $Id: xxGraphMode.cc 2832 2007-12-01 01:05:01Z flaterco $
+// $Id: xxGraphMode.cc 4497 2012-05-27 00:33:09Z flaterco $
 
 /*  xxGraphMode  Tide graphs in a window.
 
@@ -20,10 +20,14 @@
 
 #include "xtide.hh"
 #include "Graph.hh"
+#include "PixelatedGraph.hh"
 #include "RGBGraph.hh"
+#include "SVGGraph.hh"
 #include "xxPixmapGraph.hh"
 #include "xxGraphMode.hh"
-#include "xxSimplePrompt.hh"
+#include "xxHorizDialog.hh"
+#include "xxMultiChoice.hh"
+#include "xxReconfigurable.hh"
 #include "xxFilename.hh"
 
 
@@ -40,7 +44,7 @@ detail.\n\
 \n\
 The options menu supplies the following commands:\n\
 \n\
-  Save:  Use this to write the tide predictions to a PNG image file.\n\
+  Save:  Use this to write the tide predictions to an image file.\n\
 \n\
   Set Mark:  This is used to set the 'mark level' for tide predictions.\n\
   The prediction window will show the times when the tide level crosses\n\
@@ -98,9 +102,26 @@ const Interval xxGraphMode::increment() const {
 }
 
 
+void xxGraphMode::updatePixmap (bool resized) {
+  if (xxX::displaySucks) {
+    if (resized)
+      graph = std::auto_ptr<xxPixmapGraph> (new xxPixmapGraph (curGraphWidth,
+							      curGraphHeight));
+    graph->drawTides (_station.get(), t);
+  } else {
+    RGBGraph g (curGraphWidth, curGraphHeight);
+    g.drawTides (_station.get(), t);
+    if (havePixmap)
+      XFreePixmap (xxX::display, pixmap);
+    havePixmap = true;
+    pixmap = xxX::makePixmap (curGraphWidth, curGraphHeight, g.rgb);
+  }
+}
+
+
 void xxGraphMode::forward() {
   t += increment() * (scrollFraction * curGraphWidth);
-  graph->drawTides (_station.get(), t);
+  updatePixmap (false);
   draw();
 }
 
@@ -115,7 +136,7 @@ static void xxGraphModebackwardCallback (Widget w unusedParameter,
 
 void xxGraphMode::backward() {
   t -= increment() * (scrollFraction * curGraphWidth);
-  graph->drawTides (_station.get(), t);
+  updatePixmap (false);
   draw();
 }
 
@@ -156,18 +177,38 @@ void xxGraphMode::resize (Dimension newHeight, Dimension newWidth) {
 
 
 void xxGraphMode::save () {
-  (void) new xxFilename (*popup, *this, "tides.png");
+  (void) new xxFilename (*popup, *this, false);
 }
 
 
-void xxGraphMode::save (const Dstr &filename) {
-  if ((Global::PNGFile = fopen (filename.aschar(), "wb"))) {
-    RGBGraph g (curGraphWidth, curGraphHeight);
-    g.drawTides (_station.get(), t);
-    g.writeAsPNG (Global::writePNGToFile);
-    fclose (Global::PNGFile);
-  } else
-    Global::cantOpenFile (filename, Error::nonfatal);
+void xxGraphMode::save (const Dstr &filename, Format::Format form) {
+  switch (form) {
+  case Format::PNG:
+    if ((Global::PNGFile = fopen (filename.aschar(), "wb"))) {
+      RGBGraph g (curGraphWidth, curGraphHeight);
+      g.drawTides (_station.get(), t);
+      g.writeAsPNG (Global::writePNGToFile);
+      fclose (Global::PNGFile);
+    } else
+      Global::cantOpenFile (filename, Error::nonfatal);
+    break;
+  case Format::SVG:
+    {
+      FILE *fp;
+      if ((fp = fopen (filename.aschar(), "w"))) {
+	Dstr text;
+	SVGGraph g (curGraphWidth, curGraphHeight);
+	g.drawTides (_station.get(), t);
+	g.print (text);
+	fprintf (fp, "%s", text.aschar());
+	fclose (fp);
+      } else
+	Global::cantOpenFile (filename, Error::nonfatal);
+    }
+    break;
+  default:
+    assert (false);
+  }
 }
 
 
@@ -187,6 +228,7 @@ xxGraphMode::xxGraphMode (const xxWidget &shell,
 
 
 void xxGraphMode::construct () {
+  havePixmap = false;
   Dstr title (_station->name);
   title += " (Graph)";
   setTitle (title);
@@ -209,7 +251,7 @@ void xxGraphMode::construct () {
     {XtNinternalWidth, (XtArgVal)0}
   };
   {
-    Widget labelWidget = XtCreateManagedWidget ("",
+    Widget labelWidget = xxX::createXtWidget ("",
       labelWidgetClass, container->widget(), labelArgs, 6);
     label = xxX::wrap (labelWidget);
     XtAddEventHandler (labelWidget, ExposureMask, False,
@@ -220,13 +262,13 @@ void xxGraphMode::construct () {
     {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]}
   };
   {
-    Widget buttonWidget = XtCreateManagedWidget ("Backward",
+    Widget buttonWidget = xxX::createXtWidget ("Backward",
       repeaterWidgetClass, container->widget(), buttonArgs, 2);
     XtAddCallback (buttonWidget, XtNcallback, xxGraphModebackwardCallback,
      (XtPointer)this);
     backwardButton = xxX::wrap (buttonWidget);
   }{
-    Widget buttonWidget = XtCreateManagedWidget ("Forward",
+    Widget buttonWidget = xxX::createXtWidget ("Forward",
       repeaterWidgetClass, container->widget(), buttonArgs, 2);
     XtAddCallback (buttonWidget, XtNcallback, xxGraphModeforwardCallback,
      (XtPointer)this);
@@ -253,9 +295,7 @@ void xxGraphMode::construct () {
 
 
 void xxGraphMode::redraw() {
-  graph = std::auto_ptr<xxPixmapGraph> (new xxPixmapGraph (curGraphWidth,
-							   curGraphHeight));
-  graph->drawTides (_station.get(), t);
+  updatePixmap (true);
   Arg labelArgs[2] =  {
     {XtNheight, (XtArgVal)curGraphHeight},
     {XtNwidth, (XtArgVal)curGraphWidth}
@@ -266,9 +306,9 @@ void xxGraphMode::redraw() {
 
 
 void xxGraphMode::draw (int x, int y, int width, int height) {
-  if (graph.get() && _isRealized) {
+  if ((havePixmap || graph.get()) && _isRealized) {
     XCopyArea (xxX::display,
-	       graph->pixmap,
+	       (havePixmap ? pixmap : graph->pixmap),
 	       XtWindow(label->widget()),
 	       xxX::textGC,
 	       x,
@@ -296,6 +336,8 @@ void xxGraphMode::draw () {
 
 xxGraphMode::~xxGraphMode() {
   unrealize();
+  if (havePixmap)
+    XFreePixmap (xxX::display, pixmap);
 }
 
 
