@@ -1,4 +1,4 @@
-// $Id: xxRoot.cc 3696 2010-06-27 20:47:02Z flaterco $
+// $Id: xxRoot.cc 4501 2012-05-27 17:41:58Z flaterco $
 
 /*  xxRoot  XTide "root" window (control panel, top-level logic)
 
@@ -21,6 +21,7 @@
 #include "xtide.hh"
 #include "xxGraphMode.hh"
 #include "xxDisclaimer.hh"
+#include "xxReconfigurable.hh"
 #include "xxLocationList.hh"
 #include "xxGlobe.hh"
 #include "xxMap.hh"
@@ -32,6 +33,8 @@
 #include "xxTextMode.hh"
 
 
+static const unsigned controlPanelInitialHeight (900U);
+
 // Completely wild guess of how much room is needed by the window dressing on
 // an xxHelpBox (pixels).
 static const unsigned windowDressingWidth (32U);
@@ -143,6 +146,26 @@ void xxRoot::apply (ApplyProtocol protocol) {
 	  assert (false);
 	}
 	break;
+      case Configurable::gsInterp:
+	switch (choice) {
+	case 0:
+	  configurable.isNull = false;
+	  configurable.c = 'd';
+	  break;
+	case 1:
+	  configurable.isNull = false;
+	  configurable.c = 'l';
+	  break;
+	case 2:
+	  configurable.isNull = false;
+	  configurable.c = 's';
+	  break;
+	case 3:
+	  break;
+	default:
+	  assert (false);
+	}
+	break;
       default:
         assert (false);
       }
@@ -165,6 +188,7 @@ void xxRoot::apply (ApplyProtocol protocol) {
 	switch (configurable.interpretation) {
 	case Configurable::posDoubleInterp:
 	case Configurable::nonnegativeDoubleInterp:
+	case Configurable::opacityDoubleInterp:
 	  {
 	    Global::GetDoubleReturn gdr (Global::getDouble (
                                                    value,
@@ -183,6 +207,12 @@ void xxRoot::apply (ApplyProtocol protocol) {
             }
 	  }
 	  break;
+        case Configurable::textInterp:
+	  if (value.length()) {
+	    configurable.isNull = false;
+	    configurable.s = value;
+	  }
+	  break;
 	case Configurable::colorInterp:
 	  if (value.length()) {
 	    // Sanity check color
@@ -319,7 +349,7 @@ xxRoot::xxRoot (int argc, char **argv):
   Global::settings.applyUserDefaults();
   Global::settings.applyCommandLine (argc, argv);
   Global::settings.fixUpDeprecatedSettings();
-  xxX::installColors();
+  xxX::connectPostSettingsHook (popup->widget());
 
   Global::setErrorCallback (&errorCallback);
 
@@ -337,38 +367,40 @@ xxRoot::xxRoot (int argc, char **argv):
       {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]},
       {XtNvSpace, (XtArgVal)0}
     };
-    Widget formWidget = XtCreateManagedWidget ("", formWidgetClass,
+    Widget formWidget = xxX::createXtWidget ("", formWidgetClass,
       popup->widget(), formArgs, 3);
     container = xxX::wrap (formWidget);
   }{
     Arg labelArgs[6] =  {
       {XtNbackground, (XtArgVal)xxX::pixels[Colors::background]},
       {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]},
-      {XtNleft, (XtArgVal)XawChainLeft},
-      {XtNright, (XtArgVal)XawChainLeft},
-      {XtNtop, (XtArgVal)XawChainTop},
-      {XtNbottom, (XtArgVal)XawChainTop}
+      {(char*)XtNleft, (XtArgVal)XawChainLeft},
+      {(char*)XtNright, (XtArgVal)XawChainLeft},
+      {(char*)XtNtop, (XtArgVal)XawChainTop},
+      {(char*)XtNbottom, (XtArgVal)XawChainTop}
     };
-    Widget labelWidget = XtCreateManagedWidget (
+    Widget labelWidget = xxX::createXtWidget (
       "-------------- XTide Control Panel --------------",
       labelWidgetClass, container->widget(), labelArgs, 6);
     label = new xxWidget (labelWidget);
   }{
-    Arg vpArgs[9] =  {
+    Arg vpArgs[10] =  {
+      {XtNheight, (XtArgVal)controlPanelInitialHeight},
       {XtNbackground, (XtArgVal)xxX::pixels[Colors::background]},
       {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]},
       {XtNallowVert, (XtArgVal)1},
       {XtNforceBars, (XtArgVal)1},
-      {XtNfromVert, (XtArgVal)label->widget()},
-      {XtNleft, (XtArgVal)XawChainLeft},
-      {XtNright, (XtArgVal)XawChainRight},
-      {XtNtop, (XtArgVal)XawChainTop},
-      {XtNbottom, (XtArgVal)XawChainBottom}
+      {(char*)XtNfromVert, (XtArgVal)label->widget()},
+      {(char*)XtNleft, (XtArgVal)XawChainLeft},
+      {(char*)XtNright, (XtArgVal)XawChainRight},
+      {(char*)XtNtop, (XtArgVal)XawChainTop},
+      {(char*)XtNbottom, (XtArgVal)XawChainBottom}
     };
 
-    Widget viewportWidget = XtCreateManagedWidget ("", viewportWidgetClass,
-      container->widget(), vpArgs, 9);
+    Widget viewportWidget = xxX::createXtWidget ("", viewportWidgetClass,
+      container->widget(), vpArgs, 10);
     viewport = new xxWidget (viewportWidget);
+    xxX::fixBorder (viewportWidget);
 
     require (scrollbarWidget = XtNameToWidget (viewportWidget, "vertical"));
     Arg sbArgs[2] = {
@@ -376,6 +408,7 @@ xxRoot::xxRoot (int argc, char **argv):
       {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]}
     };
     XtSetValues (scrollbarWidget, sbArgs, 2);
+    setRudeness (scrollbarWidget);
   }{
     Arg boxArgs[4] =  {
       {XtNbackground, (XtArgVal)xxX::pixels[Colors::background]},
@@ -383,7 +416,7 @@ xxRoot::xxRoot (int argc, char **argv):
       {XtNvSpace, (XtArgVal)0},
       {XtNorientation, (XtArgVal)XtorientVertical}
     };
-    Widget boxWidget = XtCreateManagedWidget ("", boxWidgetClass,
+    Widget boxWidget = xxX::createXtWidget ("", boxWidgetClass,
       viewport->widget(), boxArgs, 4);
     viewBox = new xxWidget (boxWidget);
   }
@@ -421,6 +454,12 @@ xxRoot::xxRoot (int argc, char **argv):
                                     "(blank)",
                                     NULL};
 
+  static constString gsChoices[] = {"d",
+				    "l",
+				    "s",
+				    "(blank)",
+				    NULL};
+
   // Create all of the dialogs.
   for (ConfigurablesMap::iterator it = settings.begin();
        it != settings.end();
@@ -455,6 +494,7 @@ xxRoot::xxRoot (int argc, char **argv):
 	  break;
 	case Configurable::posDoubleInterp:
 	case Configurable::nonnegativeDoubleInterp:
+	case Configurable::opacityDoubleInterp:
 	  {
 	    char temp[80];
 	    if (configurable.isNull)
@@ -483,9 +523,10 @@ xxRoot::xxRoot (int argc, char **argv):
 						firstChoice);
 	  }
 	  break;
-	case Configurable::colorInterp:     // deliberate fall through
-	case Configurable::eventMaskInterp: // deliberate fall through
+	case Configurable::colorInterp:
+	case Configurable::eventMaskInterp:
 	case Configurable::timeFormatInterp:
+	case Configurable::textInterp:    
 	  dialogs[it->first] = new xxHorizDialog (
                            *viewBox,
 			   configurable.caption.aschar(),
@@ -509,6 +550,24 @@ xxRoot::xxRoot (int argc, char **argv):
 						firstChoice);
 	  }
 	  break;
+	case Configurable::gsInterp:
+	  {
+	    unsigned firstChoice;
+	    if (configurable.isNull)
+	      firstChoice = 3;
+	    else if (configurable.c == 'd')
+	      firstChoice = 0;
+	    else if (configurable.c == 'l')
+	      firstChoice = 1;
+	    else
+	      firstChoice = 2;
+	    dialogs[it->first] = new xxMultiChoice (
+                                                *viewBox,
+						configurable.caption.aschar(),
+						gsChoices,
+						firstChoice);
+	  }
+	  break;
 	default:
 	  assert (false);
 	}
@@ -522,13 +581,13 @@ xxRoot::xxRoot (int argc, char **argv):
       {XtNcolormap, (XtArgVal)xxX::colormap},
       {XtNbackground, (XtArgVal)xxX::pixels[Colors::button]},
       {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]},
-      {XtNfromVert, (XtArgVal)viewport->widget()},
-      {XtNleft, (XtArgVal)XawChainLeft},
-      {XtNright, (XtArgVal)XawChainLeft},
-      {XtNtop, (XtArgVal)XawChainBottom},
-      {XtNbottom, (XtArgVal)XawChainBottom}
+      {(char*)XtNfromVert, (XtArgVal)viewport->widget()},
+      {(char*)XtNleft, (XtArgVal)XawChainLeft},
+      {(char*)XtNright, (XtArgVal)XawChainLeft},
+      {(char*)XtNtop, (XtArgVal)XawChainBottom},
+      {(char*)XtNbottom, (XtArgVal)XawChainBottom}
     };
-    Widget buttonWidget = XtCreateManagedWidget ("Apply", commandWidgetClass,
+    Widget buttonWidget = xxX::createXtWidget ("Apply", commandWidgetClass,
       container->widget(), applyArgs, 9);
     XtAddCallback (buttonWidget, XtNcallback, applyCallback,
      (XtPointer)this);
@@ -539,14 +598,14 @@ xxRoot::xxRoot (int argc, char **argv):
       {XtNcolormap, (XtArgVal)xxX::colormap},
       {XtNbackground, (XtArgVal)xxX::pixels[Colors::button]},
       {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]},
-      {XtNfromVert, (XtArgVal)viewport->widget()},
-      {XtNfromHoriz, (XtArgVal)applyButton->widget()},
-      {XtNleft, (XtArgVal)XawChainLeft},
-      {XtNright, (XtArgVal)XawChainLeft},
-      {XtNtop, (XtArgVal)XawChainBottom},
-      {XtNbottom, (XtArgVal)XawChainBottom}
+      {(char*)XtNfromVert, (XtArgVal)viewport->widget()},
+      {(char*)XtNfromHoriz, (XtArgVal)applyButton->widget()},
+      {(char*)XtNleft, (XtArgVal)XawChainLeft},
+      {(char*)XtNright, (XtArgVal)XawChainLeft},
+      {(char*)XtNtop, (XtArgVal)XawChainBottom},
+      {(char*)XtNbottom, (XtArgVal)XawChainBottom}
     };
-    Widget buttonWidget = XtCreateManagedWidget ("Save", commandWidgetClass,
+    Widget buttonWidget = xxX::createXtWidget ("Save", commandWidgetClass,
       container->widget(), saveArgs, 10);
     XtAddCallback (buttonWidget, XtNcallback, saveCallback,
      (XtPointer)this);
@@ -557,14 +616,14 @@ xxRoot::xxRoot (int argc, char **argv):
       {XtNcolormap, (XtArgVal)xxX::colormap},
       {XtNbackground, (XtArgVal)xxX::pixels[Colors::button]},
       {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]},
-      {XtNfromVert, (XtArgVal)viewport->widget()},
-      {XtNfromHoriz, (XtArgVal)saveButton->widget()},
-      {XtNleft, (XtArgVal)XawChainLeft},
-      {XtNright, (XtArgVal)XawChainLeft},
-      {XtNtop, (XtArgVal)XawChainBottom},
-      {XtNbottom, (XtArgVal)XawChainBottom}
+      {(char*)XtNfromVert, (XtArgVal)viewport->widget()},
+      {(char*)XtNfromHoriz, (XtArgVal)saveButton->widget()},
+      {(char*)XtNleft, (XtArgVal)XawChainLeft},
+      {(char*)XtNright, (XtArgVal)XawChainLeft},
+      {(char*)XtNtop, (XtArgVal)XawChainBottom},
+      {(char*)XtNbottom, (XtArgVal)XawChainBottom}
     };
-    Widget buttonWidget = XtCreateManagedWidget ("Dismiss", commandWidgetClass,
+    Widget buttonWidget = xxX::createXtWidget ("Dismiss", commandWidgetClass,
       container->widget(), buttonArgs, 10);
     XtAddCallback (buttonWidget, XtNcallback, dismissCallback,
      (XtPointer)this);
@@ -575,14 +634,14 @@ xxRoot::xxRoot (int argc, char **argv):
       {XtNcolormap, (XtArgVal)xxX::colormap},
       {XtNbackground, (XtArgVal)xxX::pixels[Colors::button]},
       {XtNforeground, (XtArgVal)xxX::pixels[Colors::foreground]},
-      {XtNfromVert, (XtArgVal)viewport->widget()},
-      {XtNfromHoriz, (XtArgVal)dismissButton->widget()},
-      {XtNleft, (XtArgVal)XawChainLeft},
-      {XtNright, (XtArgVal)XawChainLeft},
-      {XtNtop, (XtArgVal)XawChainBottom},
-      {XtNbottom, (XtArgVal)XawChainBottom}
+      {(char*)XtNfromVert, (XtArgVal)viewport->widget()},
+      {(char*)XtNfromHoriz, (XtArgVal)dismissButton->widget()},
+      {(char*)XtNleft, (XtArgVal)XawChainLeft},
+      {(char*)XtNright, (XtArgVal)XawChainLeft},
+      {(char*)XtNtop, (XtArgVal)XawChainBottom},
+      {(char*)XtNbottom, (XtArgVal)XawChainBottom}
     };
-    Widget buttonWidget = XtCreateManagedWidget ("?", commandWidgetClass,
+    Widget buttonWidget = xxX::createXtWidget ("?", commandWidgetClass,
       container->widget(), buttonArgs, 10);
     XtAddCallback (buttonWidget, XtNcallback, helpCallback,
       (XtPointer)this);
@@ -591,6 +650,7 @@ xxRoot::xxRoot (int argc, char **argv):
 
   obeyMouseWheel (popup->widget());
   obeyMouseWheel (scrollbarWidget);
+  obeyMouseWheel (viewBox->widget());
 }
 
 
@@ -800,7 +860,7 @@ void xxRoot::newGlobe () {
 
 
 static bool lineTooLong (const Dstr &line, const unsigned widthLimit) {
-  return (xxX::stringWidth (xxX::defaultFontStruct, line) > widthLimit);
+  return (xxX::stringWidth (xxX::monoFontStruct, line) > widthLimit);
 }
 
 
@@ -831,7 +891,8 @@ static void wrapLongLine (Dstr &line,
 
 
 static void wrapLongLines (Dstr &verbiage) {
-  assert (WidthOfScreen(xxX::screen) > windowDressingWidth);
+  assert (WidthOfScreen(xxX::screen) > 0 &&
+          (unsigned)WidthOfScreen(xxX::screen) > windowDressingWidth);
   const unsigned widthLimit (WidthOfScreen(xxX::screen) - windowDressingWidth);
   Dstr formattedVerbiage, line;
   verbiage.getline (line);
@@ -928,7 +989,7 @@ void xxRoot::handleXEvents (HandleXEventsReturnProtocol protocol) {
 	// Newer window managers like Metacity, Compiz, KWin, Xfwm, and
 	// Openbox got the idea to deliver a stream of resize events while a
 	// window is being resized instead of just one for the final size.
-	// This was not in the contract but now we have to deal with it to
+	// This was not in the contract, but now we have to deal with it to
 	// avoid looking stupid.  Put off resizes and discard those that
 	// become redundant before we get around to them.
         procrastinatedResizes[foo.xconfigure.window] = foo;
@@ -943,5 +1004,3 @@ void xxRoot::handleXEvents (HandleXEventsReturnProtocol protocol) {
     procrastinatedResizes.clear();
   } while (protocol == loopForever);
 }
-
-// Cleanup2006 Done
